/*******************************************************************************

    OHMMETER DIDTICO



    C/ PIC12F675 E LCD (MTODO LPLEX - 3 FIOS)


    AUTOR: CLAUDIO LARIOS

    DATA:19/05/2016



    USO: MEDIO DE RESISTORES

    TRABALHA EM 2 ESCALAS MANUAIS:
    
    1) 0,00 OHMS A 9.999 OHMS (RESISTORES DE BAIXO VALOR)
    2) 0 A  999999 OHMS       (RESISTORES DE ALTO VALOR)

    IMPORTANTE: USE CAPACITOR DE 220nF JUNTO AOS PINOS 1 E 8
    POIS O CIRCUITO  SENSVEL A INTERFERENCIAS PELA ALIMENTAO

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(a)
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05


//pinos do lcd (metodo Lariosplex - 3 fios)
#bit DB76  = 0x5.0  //pino 7 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.1  //pino 6 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.5  //pino 2 E=direto RS=resistor 10k +cap 10nF ao gnd
#bit FX10k   = 0x5.4  //pino 3
#bit FX100   = 0x5.2  //pino 5
#bit tris_fx10k = 0x85.4
#bit tris_fx100 = 0x85.2
#bit sw    = 0x05.3  //pino 4 sw seleo de faixa

#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD




//variveis globais
unsigned int1 RS,f_m;
unsigned int8 cmil,dmil,umil,cent,dez,un,a;

unsigned int16 lido;
unsigned int32 res;

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}

/*******************************************************************************
        FAZ LEITURA DE UM CANAL DO CONVERSOR ANALGICO/DIGITAL
*******************************************************************************/
void get_canal(){
     lido=0;
     for(a=0;a<32;a++){//soma as 32 leituras sucessivas
     lido+=READ_ADC();// l valor da tenso de entrada do pino 5
     }
     lido/=32;//mdia das ultimas 32 leituras
   }



/*******************************************************************************
               MOSTRA NO LCD A LEITURA PROCESSADA
*******************************************************************************/
void mostra_res(){
  unsigned int1 zesq=0;


     if(!cmil&&!zesq){cmil=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(cmil+'0');//envia dezena de milhar
     if(sw&&zesq) env_lcd_d('.');//ponto
     if(!dmil&&!zesq){dmil=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(dmil+'0');//envia dezena de milhar

     if(!umil&&!zesq){umil=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(umil+'0');//envia milhar

     if(!sw&&zesq) env_lcd_d('.');//ponto

     if(!cent&&!zesq&&!sw){cent=240;}else{ zesq=1;}
     env_lcd_d(cent+'0');//envia centena
     if(sw) env_lcd_d(',');//virgula
     if(!dez&&!zesq&&!sw){dez=240;}else{zesq=1;}
     env_lcd_d(dez+'0');//envia dezena
     env_lcd_d(un+'0');//envia unidade

     env_lcd_d(' ');//espao
}



/*******************************************************************************
                 CONVERTE DE HEXA PARA DECIMAL
*******************************************************************************/

void converte_res(){

     cmil=res/100000;//obtm dezena de milhar
     res=res%100000;//coloca o resto da conta acima
     dmil=res/10000;//obtm dezena de milhar
     res=res%10000;//coloca o resto da conta acima
     umil=res/1000;//obtem a unidade de milhar
     res=res%1000;//coloca o resto da conta acima
     cent= res/100;//obtem a centena
     res=res%100;//coloca o resto da conta acima
     dez=res/10;//obtem a dezena
     un=res%10;//coloca o resto da conta acima (obtem a unidade)

}


/*******************************************************************************
                   NOME
*******************************************************************************/
void nome(){
   env_lcd_c(1);//apaga tela
   printf(env_lcd_d(" Tiny Ohmmeter "));//mostra o nome
}





/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(sAN3 | VSS_VDD);  //seleciona AN2 e 5v como referncia
 //  setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   setup_adc(ADC_CLOCK_DIV_64 );// ajusta o clock do adc
   set_adc_channel(3); //ajusta para o pino 3 (AN3)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisio=0b011100;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   nome();//envia nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));// o blog
   delay_ms(2000);
   nome();//envia nome
   /*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){

   unsigned int8 g;

/*******************************************************************************
              ROTINA DE OPERAO DO OHMIMETRO
              
   FRMULA USADA:  RX= (FX*VLIDO)/(5V- VLIDO)
   ADAPTANDO:  RX= (FX*VLIDO_HEX)/(1024-VLIDO_HEX)
*******************************************************************************/

     get_canal();// obtem resultado em 'lido'
     res=((int32)10000*lido)/(1024 - lido);//obtem valor do rx
     if(res>999999)f_m=1; else f_m=0;//testa limite de leitura
     converte_res();// ajusta para decimal

     //envia resultado para lcd
     env_lcd_c(0xc0); // 1 posico da linha 2
     printf(env_lcd_d("R= "));//envia para lcd
     
     if(f_m){//est fora de faixa?
        env_lcd_d('?');//sim, envia '?'
        for(g=0;g<13;g++)env_lcd_d(' ');// e apaga aps '?'
     }
     else{
     mostra_res(); //mostra resultado no LCD
     printf(env_lcd_d("Ohms "));// mostra nome da unidade de medida (Ohms)
     }
     delay_ms(300);// limita o nmero de apresentaes do lcd por segundo

   }//for(;;)

}//void main()
